@@ -1,4 +1,4 @@
-# $Id: frontend.py 5502 2008-01-14 16:43:34Z goodger $
+# $Id: frontend.py 7071 2011-07-05 10:13:46Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -22,6 +22,7 @@ Also exports the following functions:
   `validate_threshold`, `validate_colon_separated_string_list`,
   `validate_dependency_file`.
 * `make_paths_absolute`.
+* SettingSpec manipulation: `filter_settings_spec`.
 """
 
 __docformat__ = 'reStructuredText'
@@ -29,19 +30,15 @@ __docformat__ = 'reStructuredText'
 import os
 import os.path
 import sys
-import types
 import warnings
 import ConfigParser as CP
 import codecs
+import optparse
+from optparse import SUPPRESS_HELP
 import docutils
 import docutils.utils
 import docutils.nodes
-try:
-    import optparse
-    from optparse import SUPPRESS_HELP
-except ImportError:
-    import optik as optparse
-    from optik import SUPPRESS_HELP
+from docutils.error_reporting import locale_encoding, ErrorOutput
 
 
 def store_multiple(option, opt, value, parser, *args, **kwargs):
@@ -80,12 +77,6 @@ def validate_encoding_error_handler(setting, value, option_parser,
                                     config_parser=None, config_section=None):
     try:
         codecs.lookup_error(value)
-    except AttributeError:              # prior to Python 2.3
-        if value not in ('strict', 'ignore', 'replace', 'xmlcharrefreplace'):
-            raise (LookupError(
-                'unknown encoding error handler: "%s" (choices: '
-                '"strict", "ignore", "replace", or "xmlcharrefreplace")' % value),
-                   None, sys.exc_info()[2])
     except LookupError:
         raise (LookupError(
             'unknown encoding error handler: "%s" (choices: '
@@ -119,7 +110,7 @@ def validate_encoding_and_error_handler(
 
 def validate_boolean(setting, value, option_parser,
                      config_parser=None, config_section=None):
-    if isinstance(value, types.UnicodeType):
+    if isinstance(value, unicode):
         try:
             return option_parser.booleans[value.strip().lower()]
         except KeyError:
@@ -147,7 +138,7 @@ def validate_threshold(setting, value, option_parser,
 
 def validate_colon_separated_string_list(
     setting, value, option_parser, config_parser=None, config_section=None):
-    if isinstance(value, types.UnicodeType):
+    if isinstance(value, unicode):
         value = value.split(':')
     else:
         last = value.pop()
@@ -193,9 +184,9 @@ def make_paths_absolute(pathdict, keys, base_path=None):
     if base_path is None:
         base_path = os.getcwd()
     for key in keys:
-        if pathdict.has_key(key):
+        if key in pathdict:
             value = pathdict[key]
-            if isinstance(value, types.ListType):
+            if isinstance(value, list):
                 value = [make_one_path_absolute(base_path, path)
                          for path in value]
             elif value:
@@ -205,6 +196,35 @@ def make_paths_absolute(pathdict, keys, base_path=None):
 def make_one_path_absolute(base_path, path):
     return os.path.abspath(os.path.join(base_path, path))
 
+def filter_settings_spec(settings_spec, *exclude, **replace):
+    """Return a copy of `settings_spec` excluding/replacing some settings.
+
+    `settings_spec` is a tuple of configuration settings with a structure
+    described for docutils.SettingsSpec.settings_spec.
+
+    Optional positional arguments are names of to-be-excluded settings.
+    Keyword arguments are option specification replacements.
+    (See the html4strict writer for an example.)
+    """
+    settings = list(settings_spec)
+    # every third item is a sequence of option tuples
+    for i in range(2, len(settings), 3):
+        newopts = []
+        for opt_spec in settings[i]:
+            # opt_spec is ("<help>", [<option strings>], {<keyword args>})
+            opt_name = [opt_string[2:].replace('-', '_')
+                        for opt_string in opt_spec[1]
+                            if opt_string.startswith('--')
+                       ][0]
+            if opt_name in exclude:
+                continue
+            if opt_name in replace.keys():
+                newopts.append(replace[opt_name])
+            else:
+                newopts.append(opt_spec)
+        settings[i] = tuple(newopts)
+    return tuple(settings)
+
 
 class Values(optparse.Values):
 
@@ -225,7 +245,7 @@ class Values(optparse.Values):
             other_dict = other_dict.__dict__
         other_dict = other_dict.copy()
         for setting in option_parser.lists.keys():
-            if (hasattr(self, setting) and other_dict.has_key(setting)):
+            if (hasattr(self, setting) and setting in other_dict):
                 value = getattr(self, setting)
                 if value:
                     value += other_dict[setting]
@@ -256,8 +276,8 @@ class Option(optparse.Option):
                     new_value = self.validator(setting, value, parser)
                 except Exception, error:
                     raise (optparse.OptionValueError(
-                        'Error in option "%s":\n    %s: %s'
-                        % (opt, error.__class__.__name__, error)),
+                        'Error in option "%s":\n    %s'
+                        % (opt, ErrorString(error))),
                            None, sys.exc_info()[2])
                 setattr(values, setting, new_value)
             if self.overrides:
@@ -294,10 +314,10 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
               '0': 0, 'off': 0, 'no': 0, 'false': 0, '': 0}
     """Lookup table for boolean configuration file settings."""
 
-    if hasattr(codecs, 'backslashreplace_errors'):
-        default_error_encoding_error_handler = 'backslashreplace'
-    else:
-        default_error_encoding_error_handler = 'replace'
+    default_error_encoding = getattr(sys.stderr, 'encoding',
+                                     None) or locale_encoding or 'ascii'
+
+    default_error_encoding_error_handler = 'backslashreplace'
 
     settings_spec = (
         'General Docutils Options',
@@ -345,11 +365,11 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
          ('Disable backlinks from footnotes and citations.',
           ['--no-footnote-backlinks'],
           {'dest': 'footnote_backlinks', 'action': 'store_false'}),
-         ('Enable section numbering.  (default)',
+         ('Enable section numbering by Docutils.  (default)',
           ['--section-numbering'],
           {'action': 'store_true', 'dest': 'sectnum_xform',
            'default': 1, 'validator': validate_boolean}),
-         ('Disable section numbering.',
+         ('Disable section numbering by Docutils.',
           ['--no-section-numbering'],
           {'action': 'store_false', 'dest': 'sectnum_xform'}),
          ('Remove comment elements from the document tree.',
@@ -387,7 +407,7 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
                        'default': 4, 'metavar': '<level>',
                        'validator': validate_threshold}),
          ('Halt at the slightest problem.  Same as "--halt=info".',
-          ['--strict'], {'action': 'store_const', 'const': 'info',
+          ['--strict'], {'action': 'store_const', 'const': 1,
                          'dest': 'halt_level'}),
          ('Enable a non-zero exit status for non-halting system messages at '
           'or above <level>.  Default: 5 (disabled).',
@@ -422,14 +442,14 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
            'validator': validate_encoding_and_error_handler}),
          ('Specify error handler for unencodable output characters; '
           '"strict" (default), "ignore", "replace", '
-          '"xmlcharrefreplace", "backslashreplace" (Python 2.3+).',
+          '"xmlcharrefreplace", "backslashreplace".',
           ['--output-encoding-error-handler'],
           {'default': 'strict', 'validator': validate_encoding_error_handler}),
          ('Specify text encoding and error handler for error output.  '
-          'Default: ASCII:%s.'
-          % default_error_encoding_error_handler,
+          'Default: %s:%s.'
+          % (default_error_encoding, default_error_encoding_error_handler),
           ['--error-encoding', '-e'],
-          {'metavar': '<name[:handler]>', 'default': 'ascii',
+          {'metavar': '<name[:handler]>', 'default': default_error_encoding,
            'validator': validate_encoding_and_error_handler}),
          ('Specify the error handler for unencodable characters in '
           'error output.  Default: %s.'
@@ -437,7 +457,7 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
           ['--error-encoding-error-handler'],
           {'default': default_error_encoding_error_handler,
            'validator': validate_encoding_error_handler}),
-         ('Specify the language (as 2-letter code).  Default: en.',
+         ('Specify the language (as BCP 47 language tag).  Default: en.',
           ['--language', '-l'], {'dest': 'language_code', 'default': 'en',
                                  'metavar': '<name>'}),
          ('Write output file dependencies to <file>.',
@@ -585,7 +605,7 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
                 continue
             for section in (tuple(component.config_section_dependencies or ())
                             + (component.config_section,)):
-                if applied.has_key(section):
+                if section in applied:
                     continue
                 applied[section] = 1
                 settings.update(parser.get_section(section), self)
@@ -644,7 +664,7 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
         raise KeyError('No option with dest == %r.' % dest)
 
 
-class ConfigParser(CP.ConfigParser):
+class ConfigParser(CP.RawConfigParser):
 
     old_settings = {
         'pep_stylesheet': ('pep_html writer', 'stylesheet'),
@@ -666,13 +686,16 @@ Skipping "%s" configuration file.
 """
 
     def __init__(self, *args, **kwargs):
-        CP.ConfigParser.__init__(self, *args, **kwargs)
+        CP.RawConfigParser.__init__(self, *args, **kwargs)
 
         self._files = []
         """List of paths of configuration files read."""
 
+        self._stderr = ErrorOutput()
+        """Wrapper around sys.stderr catching en-/decoding errors"""
+
     def read(self, filenames, option_parser):
-        if type(filenames) in (types.StringType, types.UnicodeType):
+        if type(filenames) in (str, unicode):
             filenames = [filenames]
         for filename in filenames:
             try:
@@ -681,9 +704,12 @@ Skipping "%s" configuration file.
             except IOError:
                 continue
             try:
-                CP.ConfigParser.readfp(self, fp, filename)
+                if sys.version_info < (3,2):
+                    CP.RawConfigParser.readfp(self, fp, filename)
+                else:
+                    CP.RawConfigParser.read_file(self, fp, filename)
             except UnicodeDecodeError:
-                sys.stderr.write(self.not_utf8_error % (filename, filename))
+                self._stderr.write(self.not_utf8_error % (filename, filename))
                 fp.close()
                 continue
             fp.close()
@@ -699,7 +725,7 @@ Skipping "%s" configuration file.
         if not self.has_section('general'):
             self.add_section('general')
         for key, value in options.items():
-            if self.old_settings.has_key(key):
+            if key in self.old_settings:
                 section, setting = self.old_settings[key]
                 if not self.has_section(section):
                     self.add_section(section)
@@ -722,7 +748,7 @@ Skipping "%s" configuration file.
                 except KeyError:
                     continue
                 if option.validator:
-                    value = self.get(section, setting, raw=1)
+                    value = self.get(section, setting)
                     try:
                         new_value = option.validator(
                             setting, value, option_parser,
@@ -730,9 +756,10 @@ Skipping "%s" configuration file.
                     except Exception, error:
                         raise (ValueError(
                             'Error in config file "%s", section "[%s]":\n'
-                            '    %s: %s\n        %s = %s'
-                            % (filename, section, error.__class__.__name__,
-                               error, setting, value)), None, sys.exc_info()[2])
+                            '    %s\n'
+                            '        %s = %s'
+                            % (filename, section, ErrorString(error),
+                               setting, value)), None, sys.exc_info()[2])
                     self.set(section, setting, new_value)
                 if option.overrides:
                     self.set(section, option.overrides, None)
@@ -751,7 +778,7 @@ Skipping "%s" configuration file.
         section_dict = {}
         if self.has_section(section):
             for option in self.options(section):
-                section_dict[option] = self.get(section, option, raw=1)
+                section_dict[option] = self.get(section, option)
         return section_dict
 
 
