@@ -1,8 +1,6 @@
 # -*- coding: utf-8 -*-
-# Author: David Goodger
-# Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 4229 $
-# Date: $Date: 2005-12-23 00:46:16 +0100 (Fri, 23 Dec 2005) $
+# $Id: zh_tw.py 6460 2010-10-29 22:18:44Z milde $
+# Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
 # New language mappings are welcome.  Before doing a new translation, please
@@ -47,6 +45,7 @@ directives = {
       #'qa (translation required)': 'questions',
       #'faq (translation required)': 'questions',
       'meta (translation required)': 'meta',
+      'math (translation required)': 'math',
       #'imagemap (translation required)': 'imagemap',
       'image (translation required)': 'image',
       'figure (translation required)': 'figure',
@@ -93,6 +92,7 @@ roles = {
     'emphasis (translation required)': 'emphasis',
     'strong (translation required)': 'strong',
     'literal (translation required)': 'literal',
+    'math (translation required)': 'math',
     'named-reference (translation required)': 'named-reference',
     'anonymous-reference (translation required)': 'anonymous-reference',
     'footnote-reference (translation required)': 'footnote-reference',
